function [ft,realZa1,imagZa1]=IsoExp(ST,SM,LHorn,N,ang);
% This function calculates the complex normalized input acoustic 
% impedance, Za1, of a single segment finite exponential horn assembly 
% consisting of N sub-segments using the Holland-Fahey-Morfey (HFM)
% approach (JAES,39,315(1991)) to calculate the isophase 
% quasi-spherical progagation within the horn assembly.
% It outputs the frequency range of interest, ft, the real resistive
% part, realZa1, and the imaginary reactive part,imagZa1, to the
% Matlab workspace. It then plots the real resistive plot in blue 
% and the imaginary reactive part in red.
% The user must input five inputs:
%  ST - the horn throat area (cm2)
%  SM - the horn mouth area (cm2)
%  LHorn - the length of the horn (cm)
%  N - the number of sub-segments to divide the horn into
%  ang - the solid angle of radiation output desired in units of pi
%  steradians
%   ang = 0.5 ; 1/8 space (corner placement)
%   ang = 1.0 ; 1/4 space (wall placement)
%   ang = 2.0 ; 1/2 space (floor placement)
%   ang = 4.0 ; full space (open to full space)
% N.B., ang should only be entered as one of the 
%       four values 0.5, 1.0, 2.0, 4.0 (any other value defaults to 2.0)
%%%%%%%%%%%%%%%%%%%%%
% 
%
%%%%%%%%%%%%%%%%%%%%%
% IsoExp.m
% Version 1.0       %
% February 22, 2013 %
% Joe Ladish        %
% joeladis@unm.edu  %
%%%%%%%%%%%%%%%%%%%%%
%
% Define air parameters
%
% Enter air density, speed of sound 
%
rho0=0.001205; % g/cm3
c=3.44E4; % cm/sec
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Program Parameters That You Might Want to Change
%
% Frequency Range and Increment
fLow=10; % Low frequency limit
fHigh=5000; % High frequency limit
delf=1; % frequency interval of calculation
%
%
%
% Define Horn Segment z intervals 
% 
%z=[0 6 11 15 22 ... LHorn]; % Define z-boundaries of horn segments for 
%                              unequal segment lengths case
z=0:LHorn/N:LHorn; % Case of equal z-intervals, in this case LHorn/N
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Define full horn variables to use:
%
% Throat(T) and Mouth(M) variables, flare parameter and cut-off frequency
%
rM=sqrt(SM/pi); % radius of mouth in cm
mHorn=log(SM/ST)/LHorn; % horn flare parameter
fcHorn=mHorn*c/(4*pi); % not used in program, but available for printout
%
%
% HFM Horn Variables
%
%
% Define parameters needed for HFM segment area and length
%
S=@(z) ST*exp(mHorn*z); % planar area in horn
F=@(z) 1+(mHorn^2/(4*pi))*S(z); % intermediate function used in HFM model
Ac=@(z) (8*pi/mHorn^2)*(F(z)-sqrt(F(z))); % spherical 'cap' area in horn
AvgA=@(z) (S(z)+Ac(z))/2; % average of planar and 'cap' area in horn
delL=@(z) 2*(sqrt(F(z))-1)/mHorn; % length correction within horn
%
% Define Horn Segment Parameters
% 
N=length(z)-1; % This is the number of segments
Nstr=[num2str(N) ' Exp Segments'];
SegA=AvgA(z);
J=1:N;
SegL=(z(J+1)-z(J))+(delL(z(J+1))-delL(z(J)))/2; %  Jth segment length
%
S1=AvgA(0);  % 'throat' area = initial segment input spherical cap area cm2 
S99=AvgA(LHorn); % 'mouth area' = final segment output spherical cap area cm2
%
%
% Cell array/matrix approach to finding T1, T2, etc., and Ttot 
%
cTset=cell(1,N);
Ttot=FiniteExpHorn(SegA(1),SegA(2),SegL(1));
cTset{1,1}=Ttot;
for J1=2:N;    
    cTset{1,J1}=FiniteExpHorn(SegA(J1),SegA(J1+1),SegL(J1));
    Ttot=@(k) Ttot(k)*cTset{1,J1}(k);
end
%
% Piston in infinite baffle impedance choices by ang value
%   
%
% Enter output impedance of piston in infinite baffle
% Normalized acoustic impedance Zpn of 'piston in infinite baffle'
%
Zpn = @(x) (1-2*besselj(1,x)./x)+1i*2*struveh1approx(x)./x; 
%
% Choose correct output impedance based on value of ang
%
switch ang % check value of ang used to choose which case below
    case 4.0 % case for value of ang = 4.0 
    mult = sqrt(0.5);
    angstr=[num2str(4.0) ' pi (sr)']; % develop text string for later plot
% Zp = @(k) (rho0*c/S99)*Zpn(2*k*sqrt(0.5)*sqrt(S99/pi)); % ang=4.0*pi 
    case 2.0 % case for value of ang = 2.0
    mult = 1.0;
    angstr=[num2str(2.0) ' pi (sr)']; % develop text string for later plot
% Zp = @(k) (rho0*c/S99)*Zpn(2*k*sqrt(1)*sqrt(S99/pi));   % ang=2.0*pi
    case 1.0 % case for value of ang = 1.0
    mult = sqrt(2);
    angstr=[num2str(1.0) ' pi (sr)']; % develop text string for later plot
% Zp = @(k) (rho0*c/S99)*Zpn(2*k*sqrt(2)*sqrt(S99/pi));   % ang=1.0*pi 
    case 0.5 % case for value of ang = 0.5
    mult = sqrt(4);
    angstr=[num2str(0.5) ' pi (sr)']; % develop text string for later plot
% Zp = @(k) (rho0*c/S99)*Zpn(2*k*sqrt(4)*sqrt(S99/pi));   % ang=0.5*pi 
    otherwise % default case is ang = 2.0
    disp('Warning: Improper ang value input, ang = 2.0 (default) chosen')
    mult = 1.0;
    angstr=[num2str(2.0) ' pi (sr)']; % develop text string for later plot
% Zp = @(k) (rho0*c/S99)*Zpn(2*k*sqrt(1)*sqrt(S99/pi));   % ang=2.0*pi   
end
%
% Calculate output impedance using correct value of mult determined
% above by testing ang value
%
Zp = @(k) (rho0*c/S99)*Zpn(2*k*mult*sqrt(S99/pi));                                                    
%
%
% Calculate the individual Ttot matrix elements
%
Ttot11 = @(k) (Ttot(k)*[1;0]).'*[1;0];
Ttot12 = @(k) (Ttot(k)*[0;1]).'*[1;0];
Ttot21 = @(k) (Ttot(k)*[1;0]).'*[0;1];
Ttot22 = @(k) (Ttot(k)*[0;1]).'*[0;1];
%
%
% Calculate input impedance Za1(k)
%
Za1 = @(k) (Ttot11(k)*Zp(k)+Ttot12(k))/(Ttot21(k)*Zp(k)+Ttot22(k));
%
%
% Develop plot and output function for Za1
%
plotZa1=[];
for f=fLow:delf:fHigh;
plotZa1=cat(1,plotZa1,Za1(2*pi*f/c));
end
%
%
% Develop matrices realZa1 and imagZa1 used to hold Za1 data 
% that is output to Matlab workspace
%
f=fLow:delf:fHigh;
ft=f'; % column vector of frequency values
realZa1=[f;(S1/(rho0*c))*real(plotZa1)']';
imagZa1=[f;(S1/(rho0*c))*imag(plotZa1)']';
%
% Now plot real (blue plot) and imaginary (red plot) part
% of acoustic input impedance Za1 on semilog plot
%
figure
semilogx(f,(S1/(rho0*c))*real(plotZa1))
hold on
grid on
semilogx(f,(S1/(rho0*c))*imag(plotZa1),'r')
%
% Add Text to Plot Title to identify number of segments N and ang value
%
%['ang = ' angstr];
%
title({[Nstr ' IsoExp.m'],['Isophase Calculation: ang = ' angstr]})
%legend('Resistive','Reactive','Location','SouthWest')
xlabel('frequency  f')
ylabel('Normalized Input Acoustic Impedance')
% ylim([0 1.2]) % can be used to set vertical scale if so desired
xlim([fLow fHigh])
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%    Functions used above %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Horn Segment Functions
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%%%%%% Exponential Segment %%%%%%%
%
function [ Texp ] = FiniteExpHorn( S1,S2,L )
% FiniteExpHorn Calculates the Transfer Matrix for a finite exp horn
% This program calculates the acoustical impedance transfer matrix
% Texp of a finite exponential horn of input area S1, output
% area S2 and length L.

% Enter air density and speed of sound:
rho0=0.001205;
c=3.44E4;

% m parameter, cutoff frequency, etc.
m=log(S2/S1)/L;
fc=m*c/(4*pi); % not used in program, but available for printout
theta= @ (k) asin(0.5*m/k);

% Calculate transfer matrix Texp
Texp= @(k) [sqrt(S2/S1)*(cos(k*L*cos(theta(k))) - tan(theta(k))*sin(k*L*cos(theta(k))))...
    1i*rho0*c*sec(theta(k))*sin(k*L*cos(theta(k)))/sqrt(S1*S2);...
    1i*sqrt(S1*S2)*sec(theta(k))*sin(k*L*cos(theta(k)))/(rho0*c)...
    sqrt(S1/S2)*(cos(k*L*cos(theta(k))) + tan(theta(k))*sin(k*L*cos(theta(k))))];

end
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Calculate Struve Function needed for piston in baffle impedance
%
function output = struveh1approx(x)
% Struve H1 approximation obtained from paper by Aarts and Janssen (JASA,
% 2003)
%
% Robert J. McGough
% Michigan State University
% 18 Dec 2005

output = 2/pi - besselj(0, x) + (16/pi - 5) * sin(x) ./ x + (12 - 36/pi) .* (1 - cos(x)) ./ x .^2;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
end






